@@ -21,13 +21,23 @@
 
 #include "akonadi-prefix.h" // for prefix defines
 
+#include <QtCore/QCoreApplication>
 #include <QtCore/QDebug>
 #include <QtCore/QDir>
 #include <QtCore/QFileInfo>
 #include <QtCore/QProcess>
+#include <QtCore/QSettings>
 
 #include <cstdlib>
 
+#ifdef Q_OS_WINCE
+# include <winbase.h>
+# include <projects.h> // for FindFirstFlashCard
+# ifdef _MSC_VER
+#  pragma comment( lib, "note_prj" )
+# endif
+#endif
+
 static QStringList alternateExecPaths( const QString& path )
 {
   QStringList pathList;
@@ -57,14 +67,18 @@ namespace Akonadi {
 class XdgBaseDirsPrivate
 {
   public:
-    XdgBaseDirsPrivate() {}
+    XdgBaseDirsPrivate()
+    {
+    }
 
-    ~XdgBaseDirsPrivate() {}
+    ~XdgBaseDirsPrivate()
+    {
+    }
 };
 
 class XdgBaseDirsSingleton
 {
-public:
+  public:
     QString homePath( const char *variable, const char *defaultSubDir );
 
     QStringList systemPathList( const char *variable, const char *defaultDirList );
@@ -76,15 +90,17 @@ public:
     QStringList mConfigDirs;
     QStringList mDataDirs;
     QStringList mExecutableDirs;
+    QStringList mPluginDirs;
 };
 
-Q_GLOBAL_STATIC(XdgBaseDirsSingleton, instance)
+Q_GLOBAL_STATIC( XdgBaseDirsSingleton, instance )
 
 }
 
 using namespace Akonadi;
 
-XdgBaseDirs::XdgBaseDirs() : d( new XdgBaseDirsPrivate() )
+XdgBaseDirs::XdgBaseDirs()
+  : d( new XdgBaseDirsPrivate() )
 {
 }
 
@@ -98,6 +114,13 @@ QString XdgBaseDirs::homePath( const char *resource )
   if ( qstrncmp( "data", resource, 4 ) == 0 ) {
     if ( instance()->mDataHome.isEmpty() ) {
       instance()->mDataHome = instance()->homePath( "XDG_DATA_HOME", ".local/share" );
+#ifdef Q_OS_WINCE
+      WIN32_FIND_DATA lpwFlashCard;
+      if ( FindFirstFlashCard( &lpwFlashCard ) != INVALID_HANDLE_VALUE ) {
+              QString dirName = QString::fromUtf16( ( const ushort * ) lpwFlashCard.cFileName );
+              instance()->mDataHome = QDir::rootPath() + dirName;
+      }
+#endif
     }
     return instance()->mDataHome;
   } else if ( qstrncmp( "config", resource, 6 ) == 0 ) {
@@ -114,7 +137,14 @@ QStringList XdgBaseDirs::systemPathList( const char *resource )
 {
   if ( qstrncmp( "data", resource, 4 ) == 0 ) {
     if ( instance()->mDataDirs.isEmpty() ) {
+#ifdef Q_OS_WIN
+      QDir dir( QCoreApplication::applicationDirPath() );
+      dir.cdUp();
+      const QString defaultPathList = dir.absoluteFilePath( QLatin1String( "share" ) );
+      QStringList dataDirs = instance()->systemPathList( "XDG_DATA_DIRS", defaultPathList.toLocal8Bit().constData() );
+#else
       QStringList dataDirs = instance()->systemPathList( "XDG_DATA_DIRS", "/usr/local/share:/usr/share" );
+#endif
 
 #ifdef Q_OS_WIN
       const QString prefixDataDir = QLatin1String( AKONADIPREFIX "/" AKONADIDATA );
@@ -141,10 +171,35 @@ QStringList XdgBaseDirs::systemPathList( const char *resource )
 
       instance()->mDataDirs = dataDirs;
     }
+#ifdef Q_OS_WIN
+    QStringList dataDirs = instance()->mDataDirs;
+    // on Windows installation might be scattered across several directories
+    // so check if any installer providing agents has registered its base path
+    QSettings agentProviders( QSettings::SystemScope, QLatin1String( "Akonadi" ), QLatin1String( "Akonadi" ) );
+    agentProviders.beginGroup( QLatin1String( "AgentProviders" ) );
+    Q_FOREACH ( const QString &agentProvider, agentProviders.childKeys() ) {
+      const QString basePath = agentProviders.value( agentProvider ).toString();
+      if ( !basePath.isEmpty() ) {
+        const QString path = basePath + QDir::separator() + QLatin1String( "share" );
+        if ( !dataDirs.contains( path ) )
+          dataDirs << path;
+      }
+    }
+
+    return dataDirs;
+#else
     return instance()->mDataDirs;
+#endif
   } else if ( qstrncmp( "config", resource, 6 ) == 0 ) {
     if ( instance()->mConfigDirs.isEmpty() ) {
+#ifdef Q_OS_WIN
+      QDir dir( QCoreApplication::applicationDirPath() );
+      dir.cdUp();
+      const QString defaultPathList = dir.absoluteFilePath( QLatin1String( "etc" ) ) + QLatin1Char( ';' ) + dir.absoluteFilePath( QLatin1String( "share/config" ) );
+      QStringList configDirs = instance()->systemPathList( "XDG_CONFIG_DIRS", defaultPathList.toLocal8Bit().constData() );
+#else
       QStringList configDirs = instance()->systemPathList( "XDG_CONFIG_DIRS", "/etc/xdg" );
+#endif
 
 #ifdef Q_OS_WIN
       const QString prefixConfigDir = QLatin1String( AKONADIPREFIX "/" AKONADICONFIG );
@@ -165,19 +220,17 @@ QStringList XdgBaseDirs::systemPathList( const char *resource )
 
 QString XdgBaseDirs::findResourceFile( const char *resource, const QString &relPath )
 {
-  QString fullPath = homePath( resource ) + QLatin1Char('/' ) + relPath;
+  const QString fullPath = homePath( resource ) + QLatin1Char( '/' ) + relPath;
 
   QFileInfo fileInfo( fullPath );
   if ( fileInfo.exists() && fileInfo.isFile() && fileInfo.isReadable() ) {
     return fullPath;
   }
 
-  QStringList pathList = systemPathList( resource );
+  const QStringList pathList = systemPathList( resource );
 
-  QStringList::const_iterator it    = pathList.constBegin();
-  QStringList::const_iterator endIt = pathList.constEnd();
-  for ( ; it != endIt; ++it ) {
-    fileInfo = QFileInfo( *it + QLatin1Char('/' ) + relPath );
+  foreach ( const QString &path, pathList ) {
+    fileInfo = QFileInfo( path + QLatin1Char('/' ) + relPath );
     if ( fileInfo.exists() && fileInfo.isFile() && fileInfo.isReadable() ) {
       return fileInfo.absoluteFilePath();
     }
@@ -196,29 +249,56 @@ QString XdgBaseDirs::findExecutableFile( const QString &relPath, const QStringLi
       executableDirs << prefixExecutableDir;
     }
 
+    if ( QCoreApplication::instance() != 0 ) {
+      const QString appExecutableDir = QCoreApplication::instance()->applicationDirPath();
+      if ( !executableDirs.contains( appExecutableDir ) ) {
+        executableDirs << appExecutableDir;
+      }
+    }
+
     executableDirs += searchPath;
 
 #if defined(Q_OS_MAC) //krazy:exclude=cpp
-    executableDirs += QLatin1String(AKONADIBUNDLEPATH);
+    executableDirs += QLatin1String( AKONADIBUNDLEPATH );
 #endif
     qWarning( ) << "search paths: " << executableDirs;
 
     instance()->mExecutableDirs = executableDirs;
   }
 
-  QStringList::const_iterator pathIt    = instance()->mExecutableDirs.constBegin();
-  QStringList::const_iterator pathEndIt = instance()->mExecutableDirs.constEnd();
-  for ( ; pathIt != pathEndIt; ++pathIt ) {
-    QStringList fullPathList = alternateExecPaths(*pathIt + QLatin1Char( '/' ) + relPath );
+#ifdef Q_OS_WIN
+  QStringList executableDirs = instance()->mExecutableDirs;
+  // on Windows installation might be scattered across several directories
+  // so check if any installer providing agents has registered its base path
+  QSettings agentProviders( QSettings::SystemScope, QLatin1String( "Akonadi" ), QLatin1String( "Akonadi" ) );
+  agentProviders.beginGroup( QLatin1String( "AgentProviders" ) );
+  Q_FOREACH( const QString &agentProvider, agentProviders.childKeys() ) {
+    const QString basePath = agentProviders.value( agentProvider ).toString();
+    if ( !basePath.isEmpty() ) {
+      const QString path = basePath + QDir::separator() + QLatin1String( "bin" );
+      if ( !executableDirs.contains( path ) )
+        executableDirs << path;
+    }
+  }
 
-    QStringList::const_iterator it    = fullPathList.constBegin();
-    QStringList::const_iterator endIt = fullPathList.constEnd();
+  QStringList::const_iterator pathIt = executableDirs.constBegin();
+  const QStringList::const_iterator pathEndIt = executableDirs.constEnd();
+#else
+  QStringList::const_iterator pathIt = instance()->mExecutableDirs.constBegin();
+  const QStringList::const_iterator pathEndIt = instance()->mExecutableDirs.constEnd();
+#endif
+  for ( ; pathIt != pathEndIt; ++pathIt ) {
+    const QStringList fullPathList = alternateExecPaths( *pathIt + QLatin1Char( '/' ) + relPath );
+
+    QStringList::const_iterator it = fullPathList.constBegin();
+    const QStringList::const_iterator endIt = fullPathList.constEnd();
     for ( ; it != endIt; ++it ) {
       const QFileInfo fileInfo( *it );
 
       // resolve symlinks, happens eg. with Maemo optify
       if ( fileInfo.canonicalFilePath().isEmpty() )
         continue;
+
       const QFileInfo canonicalFileInfo( fileInfo.canonicalFilePath() );
 
       if ( canonicalFileInfo.exists() && canonicalFileInfo.isFile() && canonicalFileInfo.isExecutable() ) {
@@ -230,6 +310,60 @@ QString XdgBaseDirs::findExecutableFile( const QString &relPath, const QStringLi
   return QString();
 }
 
+QString XdgBaseDirs::findPluginFile( const QString &relPath, const QStringList &searchPath )
+{
+  if ( instance()->mPluginDirs.isEmpty() ) {
+    QStringList pluginDirs = instance()->systemPathList( "QT_PLUGIN_PATH", AKONADILIB ":" AKONADILIB "/qt4/plugins/:" AKONADILIB "/kde4/:" AKONADILIB "/kde4/plugins/:/usr/lib/qt4/plugins/" );
+
+    if ( QCoreApplication::instance() != 0 ) {
+      foreach ( const QString &libraryPath, QCoreApplication::instance()->libraryPaths() ) {
+        if ( !pluginDirs.contains( libraryPath ) ) {
+          pluginDirs << libraryPath;
+        }
+      }
+    }
+
+    pluginDirs += searchPath;
+
+    // fallback for users with KDE in a different prefix and not correctly set up XDG_DATA_DIRS, hi David ;-)
+    QProcess proc;
+    // ### should probably rather be --path xdg-something
+    const QStringList args = QStringList() << QLatin1String( "--path" ) << QLatin1String( "module" );
+    proc.start( QLatin1String( "kde4-config" ), args );
+    if ( proc.waitForStarted() && proc.waitForFinished() && proc.exitCode() == 0 ) {
+      proc.setReadChannel( QProcess::StandardOutput );
+      foreach ( const QString &path, splitPathList( QString::fromLocal8Bit( proc.readLine().trimmed() ) ) ) {
+        if ( !pluginDirs.contains( path ) )
+          pluginDirs.append( path );
+      }
+    }
+
+    qWarning( ) << "search paths: " << pluginDirs;
+    instance()->mPluginDirs = pluginDirs;
+  }
+
+#if defined(Q_OS_WIN) //krazy:exclude=cpp
+  const QString pluginName = relPath + QLatin1String( ".dll" );
+#else
+  const QString pluginName = relPath + QLatin1String( ".so" );
+#endif
+
+  foreach ( const QString &path, instance()->mPluginDirs ) {
+    const QFileInfo fileInfo( path + QDir::separator() + pluginName );
+
+    // resolve symlinks, happens eg. with Maemo optify
+    if ( fileInfo.canonicalFilePath().isEmpty() )
+      continue;
+
+    const QFileInfo canonicalFileInfo( fileInfo.canonicalFilePath() );
+    if ( canonicalFileInfo.exists() && canonicalFileInfo.isFile() ) {
+      return canonicalFileInfo.absoluteFilePath();
+    }
+  }
+
+  return QString();
+}
+
 QString XdgBaseDirs::findResourceDir( const char *resource, const QString &relPath )
 {
   QString fullPath = homePath( resource ) + QLatin1Char( '/' ) + relPath;
@@ -239,12 +373,8 @@ QString XdgBaseDirs::findResourceDir( const char *resource, const QString &relPa
     return fullPath;
   }
 
-  QStringList pathList = systemPathList( resource );
-
-  QStringList::const_iterator it    = pathList.constBegin();
-  QStringList::const_iterator endIt = pathList.constEnd();
-  for ( ; it != endIt; ++it ) {
-    fileInfo = QFileInfo( *it + QLatin1Char('/' ) + relPath );
+  foreach ( const QString &path, systemPathList( resource ) ) {
+    fileInfo = QFileInfo( path + QLatin1Char( '/' ) + relPath );
     if ( fileInfo.exists() && fileInfo.isDir() && fileInfo.isReadable() ) {
       return fileInfo.absoluteFilePath();
     }
@@ -257,19 +387,15 @@ QStringList XdgBaseDirs::findAllResourceDirs( const char *resource, const QStrin
 {
   QStringList resultList;
 
-  QString fullPath = homePath( resource ) + QLatin1Char('/' ) + relPath;
+  const QString fullPath = homePath( resource ) + QLatin1Char( '/' ) + relPath;
 
   QFileInfo fileInfo( fullPath );
   if ( fileInfo.exists() && fileInfo.isDir() && fileInfo.isReadable() ) {
     resultList << fileInfo.absoluteFilePath();
   }
 
-  QStringList pathList = systemPathList( resource );
-
-  QStringList::const_iterator it    = pathList.constBegin();
-  QStringList::const_iterator endIt = pathList.constEnd();
-  for ( ; it != endIt; ++it ) {
-    fileInfo = QFileInfo( *it + QLatin1Char('/' ) + relPath );
+  foreach ( const QString &path, systemPathList( resource ) ) {
+    fileInfo = QFileInfo( path + QLatin1Char( '/' ) + relPath );
     if ( fileInfo.exists() && fileInfo.isDir() && fileInfo.isReadable() ) {
       resultList << fileInfo.absoluteFilePath();
     }
@@ -280,7 +406,7 @@ QStringList XdgBaseDirs::findAllResourceDirs( const char *resource, const QStrin
 
 QString XdgBaseDirs::saveDir( const char *resource, const QString &relPath )
 {
-  QString fullPath = homePath( resource ) + QLatin1Char('/' ) + relPath;
+  const QString fullPath = homePath( resource ) + QLatin1Char( '/' ) + relPath;
 
   QFileInfo fileInfo( fullPath );
   if ( fileInfo.exists() ) {
@@ -316,11 +442,12 @@ QString XdgBaseDirs::akonadiConfigFile( const QString &file, FileAccessMode open
 {
   const QString akonadiDir = QLatin1String( "akonadi" );
 
-  QString savePath = saveDir( "config", akonadiDir ) + QLatin1Char( '/' ) + file;
+  const QString savePath = saveDir( "config", akonadiDir ) + QLatin1Char( '/' ) + file;
 
-  if ( openMode == WriteOnly ) return savePath;
+  if ( openMode == WriteOnly )
+    return savePath;
 
-  QString path = findResourceFile( "config", akonadiDir + QLatin1Char( '/' ) + file );
+  const QString path = findResourceFile( "config", akonadiDir + QLatin1Char( '/' ) + file );
 
   if ( path.isEmpty() ) {
     return savePath;
