@@ -20,8 +20,11 @@
 
 #include "agentmanager.h"
 
+#include "agentprocessinstance.h"
+#include "agentthreadinstance.h"
 #include "agentmanageradaptor.h"
 #include "agentmanagerinternaladaptor.h"
+#include "agentserverinterface.h"
 #include "processcontrol.h"
 #include "serverinterface.h"
 #include "libs/xdgbasedirs_p.h"
@@ -32,7 +35,9 @@
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
+#ifndef QT_NO_DEBUG
 #include <QtCore/QFileSystemWatcher>
+#endif
 #include <QtCore/QSettings>
 #include <QtDBus/QDBusConnection>
 #include <QtDBus/QDBusError>
@@ -43,15 +48,15 @@
 using Akonadi::ProcessControl;
 
 AgentManager::AgentManager( QObject *parent )
-  : QObject( parent ),
-  mAgentWatcher( new QFileSystemWatcher( this ) )
+  : QObject( parent )
+#ifndef QT_NO_DEBUG
+  , mAgentWatcher( new QFileSystemWatcher( this ) )
+#endif
 {
   new AgentManagerAdaptor( this );
   new AgentManagerInternalAdaptor( this );
   QDBusConnection::sessionBus().registerObject( "/AgentManager", this );
 
-  mTracer = new org::freedesktop::Akonadi::Tracer( "org.freedesktop.Akonadi", "/tracing", QDBusConnection::sessionBus(), this );
-
   connect( QDBusConnection::sessionBus().interface(), SIGNAL( serviceOwnerChanged( const QString&, const QString&, const QString& ) ),
            this, SLOT( serviceOwnerChanged( const QString&, const QString&, const QString& ) ) );
 
@@ -62,7 +67,13 @@ AgentManager::AgentManager( QObject *parent )
   connect( mStorageController, SIGNAL(unableToStart()), SLOT(serverFailure()) );
   mStorageController->start( "akonadiserver", QStringList(), Akonadi::ProcessControl::RestartOnCrash );
 
+  mAgentServer = new Akonadi::ProcessControl;
+  connect( mAgentServer, SIGNAL(unableToStart()), SLOT(agentServerFailure()) );
+  mAgentServer->start( "akonadi_agent_server", QStringList(), Akonadi::ProcessControl::RestartOnCrash );
+
+#ifndef QT_NO_DEBUG
   connect( mAgentWatcher, SIGNAL(fileChanged(QString)), SLOT(agentExeChanged(QString)) );
+#endif
 }
 
 void AgentManager::continueStartup()
@@ -77,8 +88,9 @@ void AgentManager::continueStartup()
   foreach ( const AgentType &info, mAgents )
     emit agentTypeAdded( info.identifier );
 
-  QStringList pathList = pluginInfoPathList();
+  const QStringList pathList = pluginInfoPathList();
 
+#ifndef QT_NO_DEBUG
   foreach ( const QString &path, pathList ) {
     QFileSystemWatcher *watcher = new QFileSystemWatcher( this );
     watcher->addPath( path );
@@ -86,6 +98,7 @@ void AgentManager::continueStartup()
     connect( watcher, SIGNAL( directoryChanged( const QString& ) ),
              this, SLOT( updatePluginInfos() ) );
   }
+#endif
 
   load();
   foreach ( const AgentType &info, mAgents )
@@ -117,8 +130,17 @@ void AgentManager::cleanup()
                                            QDBusConnection::sessionBus(), this );
   serverIface->quit();
 
+  mAgentServer->setCrashPolicy( ProcessControl::StopOnCrash );
+  org::freedesktop::Akonadi::AgentServer *agentServerIface =
+      new org::freedesktop::Akonadi::AgentServer( "org.freedesktop.Akonadi.AgentServer",
+                                                  "/AgentServer", QDBusConnection::sessionBus(), this );
+  agentServerIface->quit();
+
   delete mStorageController;
   mStorageController = 0;
+
+  delete mAgentServer;
+  mAgentServer = 0;
 }
 
 QStringList AgentManager::agentTypes() const
@@ -166,6 +188,21 @@ QStringList AgentManager::agentCapabilities( const QString &identifier ) const
   return mAgents.value( identifier ).capabilities;
 }
 
+AgentInstance::Ptr AgentManager::createAgentInstance( const AgentType &info )
+{
+  switch ( info.launchMethod ) {
+  case AgentType::Server:
+    return AgentInstance::Ptr( new Akonadi::AgentThreadInstance( this ) );
+  case AgentType::Launcher: // Fall through
+  case AgentType::Process:
+    return AgentInstance::Ptr( new Akonadi::AgentProcessInstance( this ) );
+  default:
+    Q_ASSERT_X( false, "AgentManger::createAgentInstance", "Unhandled AgentType::LaunchMethod case" );
+  }
+
+  return AgentInstance::Ptr();
+}
+
 QString AgentManager::createAgentInstance( const QString &identifier )
 {
   if ( !checkAgentExists( identifier ) )
@@ -173,16 +210,14 @@ QString AgentManager::createAgentInstance( const QString &identifier )
   AgentType agentInfo = mAgents.value( identifier );
   mAgents[ identifier ].instanceCounter++;
 
-
-  AgentInstance::Ptr instance( new AgentInstance( this ) );
+  AgentInstance::Ptr instance = createAgentInstance( agentInfo );
   if ( agentInfo.capabilities.contains( AgentType::CapabilityUnique ) )
     instance->setIdentifier( identifier );
   else
     instance->setIdentifier( QString::fromLatin1( "%1_%2" ).arg( identifier, QString::number( agentInfo.instanceCounter ) ) );
 
   if ( mAgentInstances.contains( instance->identifier() ) ) {
-    mTracer->warning( QLatin1String("AgentManager::createAgentInstance"),
-                      QString::fromLatin1( "Cannot create another instance of agent '%1'." ).arg( identifier ) );
+    akError() << Q_FUNC_INFO << "Cannot create another instance of agent" << identifier;
     return QString();
   }
 
@@ -195,7 +230,7 @@ QString AgentManager::createAgentInstance( const QString &identifier )
     return QString();
 
   mAgentInstances.insert( instance->identifier(), instance );
-  registerAgentAtServer( instance, agentInfo );
+  registerAgentAtServer( instance->identifier(), agentInfo );
   save();
 
   return instance->identifier();
@@ -204,8 +239,7 @@ QString AgentManager::createAgentInstance( const QString &identifier )
 void AgentManager::removeAgentInstance( const QString &identifier )
 {
   if ( !mAgentInstances.contains( identifier ) ) {
-    mTracer->warning( QLatin1String( "AgentManager::removeAgentInstance" ),
-                      QString( "Agent instance with identifier '%1' does not exist" ).arg( identifier ) );
+    akError() << Q_FUNC_INFO << "Agent instance with identifier" << identifier << "does not exist";
     return;
   }
 
@@ -213,14 +247,13 @@ void AgentManager::removeAgentInstance( const QString &identifier )
   if ( instance->hasAgentInterface() ) {
     instance->cleanup();
   } else {
-    mTracer->error( QLatin1String( "AgentManager::removeAgentInstance" ),
-                    QString( "Agent instance '%1' has no interface!" ).arg( identifier ) );
+    akError() << Q_FUNC_INFO << "Agent instance" << identifier << "has no interface!";
   }
 
   mAgentInstances.remove( identifier );
 
   save();
-  
+
   org::freedesktop::Akonadi::ResourceManager * resmanager = new org::freedesktop::Akonadi::ResourceManager( QLatin1String("org.freedesktop.Akonadi"), QLatin1String("/ResourceManager"), QDBusConnection::sessionBus(), this );
   resmanager->removeResourceInstance(instance->identifier());
 
@@ -234,14 +267,20 @@ void AgentManager::removeAgentInstance( const QString &identifier )
 
   preProcessorManager.unregisterInstance( instance->identifier() );
 
+  if ( instance->hasAgentInterface() ) {
+    qDebug() << "AgentManager::removeAgentInstance: calling instance->quit()";
+    instance->quit();
+  } else {
+    akError() << Q_FUNC_INFO << "Agent instance" << identifier << "has no interface!";
+  }
+
   emit agentInstanceRemoved( identifier );
 }
 
 QString AgentManager::agentInstanceType( const QString &identifier )
 {
   if ( !mAgentInstances.contains( identifier ) ) {
-    mTracer->warning( QLatin1String( "AgentManager::agentInstanceType" ),
-                      QString( "Agent instance with identifier '%1' does not exist" ).arg( identifier ) );
+    akError() << Q_FUNC_INFO << "Agent instance with identifier" << identifier << "does not exist";
     return QString();
   }
 
@@ -285,7 +324,15 @@ void AgentManager::agentInstanceConfigure( const QString &identifier, qlonglong
 {
   if ( !checkAgentInterfaces( identifier, "agentInstanceConfigure" ) )
     return;
-  mAgentInstances.value( identifier )->controlInterface()->configure( windowId );
+
+  org::freedesktop::Akonadi::AgentServer agentServer( "org.freedesktop.Akonadi.AgentServer",
+                                                      "/AgentServer", QDBusConnection::sessionBus(), this );
+
+  if ( agentServer.started( identifier ) ) {
+    agentServer.agentInstanceConfigure( identifier, windowId );
+  } else {
+    mAgentInstances.value( identifier )->controlInterface()->configure( windowId );
+  }
 }
 
 bool AgentManager::agentInstanceOnline(const QString & identifier)
@@ -371,8 +418,10 @@ void AgentManager::updatePluginInfos()
 
 void AgentManager::readPluginInfos()
 {
+#ifndef QT_NO_DEBUG
   if ( !mAgentWatcher->files().isEmpty() )
     mAgentWatcher->removePaths( mAgentWatcher->files() );
+#endif
   mAgents.clear();
 
   QStringList pathList = pluginInfoPathList();
@@ -394,9 +443,7 @@ void AgentManager::readPluginInfos( const QDir& directory )
     AgentType agentInfo;
     if ( agentInfo.load( fileName, this ) ) {
       if ( mAgents.contains( agentInfo.identifier ) ) {
-        mTracer->error( QLatin1String( "AgentManager::updatePluginInfos" ),
-                        QString( "Duplicated agent identifier '%1' from file '%2'" )
-                            .arg( agentInfo.identifier, fileName ) );
+        akError() << Q_FUNC_INFO << "Duplicated agent identifier" << agentInfo.identifier << "from file" << fileName;
         continue;
       }
 
@@ -406,16 +453,20 @@ void AgentManager::readPluginInfos( const QDir& directory )
         agentInfo.capabilities.removeOne( AgentType::CapabilityAutostart );
       }
 
-      const QString executable = Akonadi::XdgBaseDirs::findExecutableFile( agentInfo.exec );
-      if ( executable.isEmpty() ) {
-        akError() << "Executable" << agentInfo.exec << "for agent" << agentInfo.identifier << "could not be found!";
-        continue;
+      if ( agentInfo.launchMethod == AgentType::Process ) {
+        const QString executable = Akonadi::XdgBaseDirs::findExecutableFile( agentInfo.exec );
+        if ( executable.isEmpty() ) {
+          akError() << "Executable" << agentInfo.exec << "for agent" << agentInfo.identifier << "could not be found!";
+          continue;
+        }
+#ifndef QT_NO_DEBUG
+        if ( !mAgentWatcher->files().contains( executable ) )
+          mAgentWatcher->addPath( executable );
+#endif
       }
 
       qDebug() << "PLUGINS inserting: " << agentInfo.identifier << agentInfo.instanceCounter << agentInfo.capabilities;
       mAgents.insert( agentInfo.identifier, agentInfo );
-      if ( !mAgentWatcher->files().contains( executable) )
-	mAgentWatcher->addPath( executable );
     }
   }
 }
@@ -447,25 +498,24 @@ void AgentManager::load()
     const QString instanceIdentifier = entries[ i ];
 
     if ( mAgentInstances.contains( instanceIdentifier ) ) {
-      mTracer->warning( QLatin1String( "AgentManager::load" ),
-                        QString( "Duplicated instance identifier '%1' found in agentsrc" ).arg( instanceIdentifier ) );
+      akError() << Q_FUNC_INFO << "Duplicated instance identifier" << instanceIdentifier << "found in agentsrc";
       continue;
     }
 
     file.beginGroup( entries[ i ] );
 
-    const QString agentType = file.value( "AgentType" ).toString();
-    if ( !mAgents.contains( agentType ) ) {
-      mTracer->warning( QLatin1String( "AgentManager::load" ),
-                        QString( "Reference to unknown agent type '%1' in agentsrc" ).arg( agentType ) );
+    const QString agentTypeStr = file.value( "AgentType" ).toString();
+    if ( !mAgents.contains( agentTypeStr ) ) {
+      akError() << Q_FUNC_INFO << "Reference to unknown agent type" << agentTypeStr << "in agentsrc";
       file.endGroup();
       continue;
     }
 
-    AgentInstance::Ptr instance( new AgentInstance( this ) );
+    AgentInstance::Ptr instance = createAgentInstance( mAgents.value( agentTypeStr ) );
     instance->setIdentifier( instanceIdentifier );
-    if ( instance->start( mAgents.value( agentType ) ) )
+    if ( instance->start( mAgents.value( agentTypeStr ) ) )
       mAgentInstances.insert( instanceIdentifier, instance );
+
     file.endGroup();
   }
 
@@ -574,11 +624,7 @@ void AgentManager::serviceOwnerChanged( const QString &name, const QString &, co
 
     if( !preProcessorManager.isValid() )
     {
-      mTracer->warning(
-          QLatin1String( "AgentManager::serviceOwnerChanged" ),
-          QString( "Could not connect to PreprocessorManager via D-Bus: %1" )
-            .arg( preProcessorManager.lastError().message() )
-        );
+      akError() << Q_FUNC_INFO <<"Could not connect to PreprocessorManager via D-Bus:" << preProcessorManager.lastError();
     } else {
       if ( newOwner.isEmpty() )
       {
@@ -658,14 +704,33 @@ void AgentManager::ensureAutoStart(const AgentType & info)
 {
   if ( !info.capabilities.contains( AgentType::CapabilityAutostart ) )
     return; // no an autostart agent
-  if ( mAgentInstances.contains( info.identifier ) )
+
+  org::freedesktop::Akonadi::AgentServer agentServer( "org.freedesktop.Akonadi.AgentServer",
+                                                      "/AgentServer", QDBusConnection::sessionBus(), this );
+    
+  if ( mAgentInstances.contains( info.identifier ) || agentServer.started( info.identifier ) )
     return; // already running
-  AgentInstance::Ptr instance( new AgentInstance( this ) );
-  instance->setIdentifier( info.identifier );
-  if ( instance->start( info ) ) {
-    mAgentInstances.insert( instance->identifier(), instance );
-    registerAgentAtServer( instance, info );
+
+  switch ( info.launchMethod ) {
+  case AgentType::Server:
+    agentServer.startAgent( info.identifier, info.identifier, info.exec );
+    registerAgentAtServer( info.identifier, info );
     save();
+    break;
+  case AgentType::Process:  // Fall through
+  case AgentType::Launcher:
+  {
+    AgentInstance::Ptr instance = createAgentInstance( info );
+    instance->setIdentifier( info.identifier );
+    if ( instance->start( info ) ) {
+      mAgentInstances.insert( instance->identifier(), instance );
+      registerAgentAtServer( instance->identifier(), info );
+      save();
+    }
+    break;
+  }
+  default:
+    Q_ASSERT_X( false, "AgentManager::ensureAutoStart", "unhandled AgentType::LaunchMethod case" );
   }
 }
 
@@ -683,14 +748,14 @@ void AgentManager::agentExeChanged(const QString & fileName)
   }
 }
 
-void AgentManager::registerAgentAtServer( const AgentInstance::Ptr &instance, const AgentType &type )
+void AgentManager::registerAgentAtServer( const QString &agentIdentifier, const AgentType &type )
 {
   if ( type.capabilities.contains( AgentType::CapabilityResource ) ) {
     boost::scoped_ptr<org::freedesktop::Akonadi::ResourceManager> resmanager(
       new org::freedesktop::Akonadi::ResourceManager( QLatin1String("org.freedesktop.Akonadi"),
                                                       QLatin1String("/ResourceManager"),
                                                       QDBusConnection::sessionBus(), this ) );
-    resmanager->addResourceInstance( instance->identifier(), type.capabilities );
+    resmanager->addResourceInstance( agentIdentifier, type.capabilities );
   }
 }
 
@@ -715,6 +780,13 @@ void AgentManager::removeSearch(quint64 resultCollectionId)
   }
 }
 
+void AgentManager::agentServerFailure()
+{
+  qDebug() << "Failed to start AgentServer!";
+  // if ( requiresAgentServer )
+  //   QCoreApplication::instance()->exit( 255 );
+}
+
 void AgentManager::serverFailure()
 {
   QCoreApplication::instance()->exit( 255 );
