@@ -27,10 +27,11 @@
 
 #include "agenttype.h"
 #include "agentinstance.h"
-#include "tracerinterface.h"
 
 class QDir;
+#ifndef QT_NO_DEBUG
 class QFileSystemWatcher;
+#endif
 
 namespace Akonadi {
   class ProcessControl;
@@ -70,8 +71,6 @@ class AgentManager : public QObject, protected QDBusContext
      */
     static QString configPath( bool writeable );
 
-    org::freedesktop::Akonadi::Tracer* tracer() const { return mTracer; }
-
   public Q_SLOTS:
     /**
      * Returns the list of identifiers of all available
@@ -262,6 +261,15 @@ class AgentManager : public QObject, protected QDBusContext
      */
     void agentInstanceStatusChanged( const QString &agentIdentifier, int status, const QString &message );
 
+    /**
+     * This signal is emitted whenever the status of an agent instance has
+     * changed.
+     *
+     * @param agentIdentifier The identifier of the agent that has changed.
+     * @param status The object that describes the status change.
+     */
+    void agentInstanceAdvancedStatusChanged( const QString &agentIdentifier, const QVariantMap &status );
+
     /**
      * This signal is emitted whenever the progress of an agent instance has
      * changed.
@@ -305,6 +313,7 @@ class AgentManager : public QObject, protected QDBusContext
     void updatePluginInfos();
     void serviceOwnerChanged( const QString&, const QString&, const QString& );
     void agentExeChanged( const QString &fileName );
+    void agentServerFailure();
     void serverFailure();
 
   private:
@@ -336,13 +345,14 @@ class AgentManager : public QObject, protected QDBusContext
      */
     void readPluginInfos( const QDir &directory );
 
+    AgentInstance::Ptr createAgentInstance( const AgentType &type );
     bool checkAgentInterfaces( const QString &identifier, const QString &method ) const;
     bool checkInstance( const QString &identifier ) const;
     bool checkResourceInterface( const QString &identifier, const QString &method ) const;
     bool checkAgentExists( const QString &identifier ) const;
     void ensureAutoStart( const AgentType &info );
     void continueStartup();
-    void registerAgentAtServer( const AgentInstance::Ptr& instance, const AgentType& type );
+    void registerAgentAtServer( const QString &agentIdentifier, const AgentType& type );
 
   private:
     /**
@@ -360,10 +370,11 @@ class AgentManager : public QObject, protected QDBusContext
      */
     QHash<QString, AgentInstance::Ptr> mAgentInstances;
 
-    org::freedesktop::Akonadi::Tracer *mTracer;
-
+    Akonadi::ProcessControl *mAgentServer;
     Akonadi::ProcessControl *mStorageController;
+#ifndef QT_NO_DEBUG
     QFileSystemWatcher *mAgentWatcher;
+#endif
 
     friend class AgentInstance;
 };
