@@ -38,10 +38,6 @@ class AgentProcessInstance : public AgentInstance
     virtual void quit();
     virtual void cleanup();
     virtual void restartWhenIdle();
-    virtual void configure( qlonglong windowId );
-
-  private Q_SLOTS:
-    void failedToStart();
 
   private:
     Akonadi::ProcessControl *mController;
