@@ -29,6 +29,10 @@
 #include <signal.h>
 #endif
 
+#ifdef Q_OS_WINCE
+#include <windows.h>
+#endif
+
 using namespace Akonadi;
 
 ProcessControl::ProcessControl( QObject *parent )
@@ -39,10 +43,7 @@ ProcessControl::ProcessControl( QObject *parent )
            this, SLOT( slotError( QProcess::ProcessError ) ) );
   connect( &mProcess, SIGNAL( finished( int, QProcess::ExitStatus ) ),
            this, SLOT( slotFinished( int, QProcess::ExitStatus ) ) );
-  connect( &mProcess, SIGNAL( readyReadStandardError() ),
-           this, SLOT( slotErrorMessages() ) );
-  connect( &mProcess, SIGNAL( readyReadStandardOutput() ),
-            this, SLOT( slotStdoutMessages() ) );
+  mProcess.setProcessChannelMode( QProcess::ForwardedChannels );
 }
 
 ProcessControl::~ProcessControl()
@@ -98,6 +99,7 @@ void ProcessControl::slotFinished( int exitCode, QProcess::ExitStatus exitStatus
       if ( !mFailedToStart && --mCrashCount >= 0 ) {
         qDebug( "Application '%s' crashed! %d restarts left.", qPrintable( mApplication ), mCrashCount );
         start();
+        emit restarted();
       } else {
         if ( mFailedToStart ) {
           qDebug( "Application '%s' failed to start!", qPrintable( mApplication ) );
@@ -115,6 +117,25 @@ void ProcessControl::slotFinished( int exitCode, QProcess::ExitStatus exitStatus
     if ( exitCode != 0 ) {
       qDebug( "ProcessControl: Application '%s' returned with exit code %d (%s)",
               qPrintable( mApplication ), exitCode, qPrintable( mProcess.errorString() ) );
+#ifdef Q_OS_WINCE
+	  // If Akonadi server crashed clean the registry key for the local socket.
+	  // Qt insertes a regitsry key for each local socket opened and checks if a new
+	  // socket is opened if some other socket is already bind, if a appication
+	  // crashes the registru key is not deleted and no local socket can be opened
+	  // so try to delete the key if it fails once
+	  if ( mApplication == "akonadiserver" && exitCode == 1) {
+	    HKEY hKey;
+        long rc = RegOpenKeyEx (HKEY_CURRENT_USER, TEXT("Software\\Trolltech\\Qt\\QLocalServer"), 0, KEY_ALL_ACCESS, &hKey);
+	    if(rc != ERROR_SUCCESS ) {
+	      qWarning() << mApplication << "Tried to open HKCU\\Software\\Trolltech\\Qt\\QLocalServer to delete Key Akonadi";
+	    } else {
+	      rc =  RegDeleteValue( hKey, TEXT("Akonadi") );
+          if(rc != ERROR_SUCCESS ) {
+            qWarning() << mApplication << "Could not delete HKCU\\Software\\Trolltech\\Qt\\QLocalServer\\Akonadi";
+		  }
+	    }
+	  }
+#endif
       if ( mPolicy == RestartOnCrash ) {
         if ( mCrashCount > 2 ) {
           qWarning() << mApplication << "crashed too often and will not be restarted!";
@@ -124,8 +145,10 @@ void ProcessControl::slotFinished( int exitCode, QProcess::ExitStatus exitStatus
         }
         ++mCrashCount;
         QTimer::singleShot( 60000, this, SLOT(resetCrashCount()) );
-        if ( !mFailedToStart ) // don't try to start an unstartable application
+        if ( !mFailedToStart ) { // don't try to start an unstartable application
           start();
+          emit restarted();
+        }
       }
     } else {
       if ( mRestartOnceOnExit ) {
@@ -183,7 +206,7 @@ void ProcessControl::start()
       qDebug();
       qDebug() << "============================================================";
       qDebug() << "ProcessControl: Suspending process" << mApplication;
-      qDebug() << "'gdb" << pid << "' to debug";
+      qDebug() << "'gdb --pid" << pid << "' to debug";
       qDebug() << "'kill -SIGCONT" << pid << "' to continue";
       qDebug() << "============================================================";
       qDebug();
@@ -193,25 +216,6 @@ void ProcessControl::start()
 #endif
 }
 
-void Akonadi::ProcessControl::slotStdoutMessages()
-{
-  mProcess.setReadChannel( QProcess::StandardOutput );
-  while ( mProcess.canReadLine() ) {
-    QString message = QString::fromUtf8( mProcess.readLine() );
-    qDebug() << mApplication << "[out]" << message;
-  }
-}
-
-void ProcessControl::slotErrorMessages()
-{
-  mProcess.setReadChannel( QProcess::StandardError );
-  while ( mProcess.canReadLine() ) {
-    QString message = QString::fromUtf8( mProcess.readLine() );
-    emit processErrorMessages( message );
-    qDebug( "[%s] %s", qPrintable( mApplication ), qPrintable( message.trimmed() ) );
-  }
-}
-
 void ProcessControl::resetCrashCount()
 {
   mCrashCount = 0;
