@@ -35,7 +35,11 @@
 
 using namespace Akonadi;
 
+#ifdef Q_OS_WINCE
+AkonadiConnection::AkonadiConnection( int socketDescriptor, QObject *parent )
+#else
 AkonadiConnection::AkonadiConnection( quintptr socketDescriptor, QObject *parent )
+#endif
     : QThread(parent)
     , m_socketDescriptor(socketDescriptor)
     , m_socket( 0 )
@@ -63,7 +67,11 @@ AkonadiConnection::~AkonadiConnection()
 
 void AkonadiConnection::run()
 {
+#ifdef Q_OS_WINCE
+    m_socket = new QTcpSocket();
+#else
     m_socket = new QLocalSocket();
+#endif
 
     if ( !m_socket->setSocketDescriptor( m_socketDescriptor ) ) {
         qWarning() << "AkonadiConnection(" << m_identifier
@@ -89,7 +97,7 @@ void AkonadiConnection::run()
     connect( m_socket, SIGNAL( disconnected() ),
              this, SLOT( slotDisconnected() ), Qt::DirectConnection );
 
-    writeOut( "* OK Akonadi Almost IMAP Server [PROTOCOL 26]");
+    writeOut( "* OK Akonadi Almost IMAP Server [PROTOCOL 27]");
 
     m_streamParser = new ImapStreamParser( m_socket );
     m_streamParser->setTracerIdentifier( m_identifier );
@@ -118,7 +126,7 @@ void AkonadiConnection::slotNewData()
       if ( tag.isEmpty() && m_streamParser->atCommandEnd() )
         continue;
       const QByteArray command = m_streamParser->readString();
-      Tracer::self()->connectionInput( m_identifier, QString::fromUtf8( tag + ' ' + command + ' ' + m_streamParser->readRemainingData() ) );
+      Tracer::self()->connectionInput( m_identifier, (tag + ' ' + command + ' ' + m_streamParser->readRemainingData()) );
       m_currentHandler = findHandlerForCommand( command );
       assert( m_currentHandler );
       connect( m_currentHandler, SIGNAL( responseAvailable( const Response & ) ),
@@ -138,8 +146,7 @@ void AkonadiConnection::slotNewData()
       } catch ( ... ) {}
     } catch ( const Akonadi::Exception &e ) {
       if ( m_currentHandler ) {
-        m_currentHandler->failureResponse( QString::fromLatin1( e.type() )
-          + QLatin1String( ": " ) + QString::fromLatin1( e.what()  ) );
+        m_currentHandler->failureResponse( QByteArray( e.type() ) + QByteArray( ": " ) + QByteArray( e.what()  ) );
       }
       try {
         m_streamParser->skipCurrentCommand();
@@ -170,7 +177,7 @@ void AkonadiConnection::writeOut( const QByteArray &data )
     m_socket->write(block);
     m_socket->waitForBytesWritten();
 
-    Tracer::self()->connectionOutput( m_identifier, QString::fromUtf8( block ) );
+    Tracer::self()->connectionOutput( m_identifier, block );
 }
 
 
@@ -217,7 +224,11 @@ void AkonadiConnection::slotConnectionStateChange( ConnectionState state )
         case Selected:
             break;
         case LoggingOut:
+#ifdef Q_OS_WINCE
+            m_socket->disconnectFromHost();
+#else
             m_socket->disconnectFromServer();
+#endif
             break;
     }
 }
