@@ -58,6 +58,15 @@ class HandlerHelper
     */
     static int itemCount( const Collection &col );
 
+    /**
+     * Queries for collection statistics.
+     * @param col The collection to query.
+     * @param count The total amount of items in this collection.
+     * @param size The size of all items in this collection.
+     * @return @c false on a query error, @c true otherwise
+     */
+    static bool itemStatistics( const Collection &col, qint64 &count, qint64 &size );
+
     /**
       Returns the amount of existing items in the given collection
       which have a given flag set.
@@ -72,12 +81,6 @@ class HandlerHelper
     */
     static int itemWithoutFlagCount( const Collection &col, const QString &flag );
 
-    /**
-      Returns the total size of all the items in the given collection.
-      @return -1 on error
-     */
-    static qint64 itemsTotalSize( const Collection &col );
-
     /**
       Parse cache policy and update the given Collection object accoordingly.
       @param changed Indicates whether or not the cache policy already available in @p col
