@@ -1,5 +1,6 @@
 /*
     Copyright (c) 2006 - 2007 Volker Krause <vkrause@kde.org>
+    Copyright (c) 2010 Michael Jansen <kde@michael-jansen>
 
     This library is free software; you can redistribute it and/or modify it
     under the terms of the GNU Library General Public License as published by
@@ -19,6 +20,7 @@
 
 #include "notificationmanager.h"
 #include "notificationmanageradaptor.h"
+#include "notificationsource.h"
 #include "tracer.h"
 #include "storage/datastore.h"
 #include "../../libs/xdgbasedirs_p.h"
@@ -27,6 +29,7 @@
 #include <QDBusConnection>
 #include <QSettings>
 
+
 using namespace Akonadi;
 
 NotificationManager* NotificationManager::mSelf = 0;
@@ -83,8 +86,44 @@ void NotificationManager::emitPendingNotifications()
   foreach ( const NotificationMessage &msg, mNotifications ) {
     Tracer::self()->signal( "NotificationManager::notify", msg.toString() );
   }
+
+  Q_FOREACH( NotificationSource *src, mNotificationSources ) {
+    src->emitNotification( mNotifications );
+  }
+
+  // backward compatibility with the old non-subcription interface
   emit notify( mNotifications );
+
   mNotifications.clear();
 }
 
+
+
+QDBusObjectPath NotificationManager::subscribe( const QString &identifier )
+{
+  NotificationSource *source = mNotificationSources.value( identifier );
+  if ( source ) {
+    // :TODO: Should this really be a warning?
+    qDebug() << "Known subscriber" << identifier << "subscribes again";
+  } else {
+    source = new NotificationSource( identifier, this );
+    mNotificationSources.insert( identifier, source );
+  }
+  return source->dbusPath();
+}
+
+
+
+void NotificationManager::unsubscribe( const QString &identifier )
+{
+  NotificationSource *source = mNotificationSources.take( identifier );
+  if ( source ) {
+    source->deleteLater();
+  } else {
+    qDebug() << "Attempt to unsubscribe unknown subscriber" << identifier;
+  }
+}
+
+
+
 #include "notificationmanager.moc"
