@@ -28,7 +28,6 @@
 
 class QSqlQuery;
 
-#include "akonadiprivate_export.h"
 #include "entities.h"
 #include "notificationcollector.h"
 
@@ -86,7 +85,7 @@ Name=/home/foo/.local/share/akonadi/akonadi.db
   - StartServer: Start the database locally just for Akonadi instead of using an existing one
   - ServerPath: Path to the server executable
 */
-class AKONADIPRIVATE_EXPORT DataStore : public QObject
+class DataStore : public QObject
 {
     Q_OBJECT
   public:
@@ -116,10 +115,10 @@ class AKONADIPRIVATE_EXPORT DataStore : public QObject
     static DataStore* self();
 
     /* --- ItemFlags ----------------------------------------------------- */
-    bool setItemFlags( const PimItem &item, const QList<Flag> &flags );
-    bool appendItemFlags( const PimItem &item, const QList<Flag> &flags, bool& flagsChanged,
+    bool setItemFlags( const PimItem &item, const QVector<Flag> &flags );
+    bool appendItemFlags( const PimItem &item, const QVector<Flag> &flags, bool& flagsChanged,
                           bool checkIfExists = true, const Collection &col = Collection() );
-    bool removeItemFlags( const PimItem &item, const QList<Flag> &flags );
+    bool removeItemFlags( const PimItem &item, const QVector<Flag> &flags );
 
     /* --- ItemParts ----------------------------------------------------- */
     bool removeItemParts( const PimItem &item, const QList<QByteArray> &parts );
@@ -148,7 +147,7 @@ class AKONADIPRIVATE_EXPORT DataStore : public QObject
     bool appendMimeType( const QString & mimetype, qint64 *insertId = 0 );
 
     /* --- PimItem ------------------------------------------------------- */
-    bool appendPimItem( QList<Part> & parts,
+    bool appendPimItem( QVector<Part> & parts,
                         const MimeType & mimetype,
                         const Collection & collection,
                         const QDateTime & dateTime,
@@ -187,17 +186,11 @@ class AKONADIPRIVATE_EXPORT DataStore : public QObject
      */
     bool unhideAllPimItems();
 
-    qint64 highestPimItemId() const;
-
     /* --- Collection attributes ------------------------------------------ */
     bool addCollectionAttribute( const Collection &col, const QByteArray &key, const QByteArray &value );
     bool removeCollectionAttribute( const Collection &col, const QByteArray &key );
 
     /* --- Helper functions ---------------------------------------------- */
-    /** Returns the id of the next PIM item that is added to the db.
-        @return possible id of the next PIM item that is added to the database
-     */
-    qint64 uidNext() const;
 
     /**
       Begins a transaction. No changes will be written to the database and
