@@ -24,7 +24,6 @@
 
 #include <QtCore/QDebug>
 #include <QtCore/QFile>
-#include <QtCore/QList>
 #include <QtCore/QPair>
 #include <QtCore/QStringList>
 #include <QtCore/QVariant>
@@ -67,6 +66,8 @@ DbInitializer::Ptr DbInitializer::createInstance(const QSqlDatabase& database, c
     return boost::shared_ptr<DbInitializer>( new DbInitializerSqlite( database, templateFile ) );
   if ( database.driverName().startsWith( QLatin1String( "QPSQL" ) ) )
     return boost::shared_ptr<DbInitializer>( new DbInitializerPostgreSql( database, templateFile ) );
+  if ( database.driverName().startsWith( QLatin1String( "QODBC" ) ) )
+    return boost::shared_ptr<DbInitializer>( new DbInitializerVirtuoso( database, templateFile ) );
   akFatal() << database.driverName() << "backend  not supported";
   return boost::shared_ptr<DbInitializer>();
 }
