@@ -45,24 +45,31 @@ class ItemRetriever
 {
   public:
     ItemRetriever( AkonadiConnection *connection );
+    ~ItemRetriever();
+
+    AkonadiConnection *connection() const;
 
     void setRetrieveParts( const QStringList &parts );
+    QStringList retrieveParts() const;
     void setRetrieveFullPayload( bool fullPayload );
     void setItemSet( const ImapSet &set, const Collection &collection = Collection() );
     void setItemSet( const ImapSet &set, bool isUid );
     void setItem( const Akonadi::Entity::Id &id );
     /** Retrieve all items in the given collection. */
     void setCollection( const Akonadi::Collection &collection, bool recursive = true );
+
     /** Retrieve all items matching the given item scope. */
     void setScope( const Scope &scope );
+    Scope scope() const;
 
     void exec();
 
   private:
-    QueryBuilder buildItemQuery() const;
-    QueryBuilder buildPartQuery() const;
+    /** Convenience method which returns the database driver name */
+    QString driverName();
+
+    QSqlQuery buildQuery() const;
 
-  private:
     ImapSet mItemSet;
     Collection mCollection;
     Scope mScope;
